/* A Rust implementation of Mike Ash's fluid solver from
 * https://mikeash.com/pyblog/fluid-simulation-for-dummies.html
 *
 * Inspired by The Coding Trains Challenge #132
 * https://www.youtube.com/watch?v=alhpH6ECFvQ
 */

extern crate piston_window;

use piston_window::*;

mod fluid;
mod tests;

fn main() {
    let size: i32 = 80;
    let scale: i32 = 5;
    let iter: i32 = 10;
    let dye_amount = 1.0;
    let diffusion = 0.2;
    let viscosity = 0.000000001;
    let dt = 0.000001;
    let mut fs: fluid::FluidSquare = fluid::FluidSquare::new(size, dt, viscosity, diffusion);
    // fs.step(iter);
    println!("Hello, world!");

    let _background = [0.1, 0.1, 0.1, 1.0];
    let mut events = Events::new(EventSettings::new());
    let mut dye = false;
    let mut vel = false;
    let mut vis = true;
    let mut prev_x: f64 = 0.0;
    let mut prev_y: f64 = 0.0;

    let mut window: PistonWindow = WindowSettings::new("Hello Piston!", [(size * scale) as u32; 2])
        .exit_on_esc(true)
        // .fullscreen(true)
        .build()
        .unwrap_or_else(|e| panic!("Failed to build PistonWindow: {}", e));

    while let Some(e) = events.next(&mut window) {
        window.draw_2d(&e, |_c, g| {
            fs.step(iter);
            clear(_background, g);
            //  clear(color::BLACK, g);
            for i in 0..size {
                for j in 0..size {
                    let x = i as i32 * scale;
                    let y = j as i32 * scale;

                    let rectangle = Rectangle::new(color::alpha(if vis {
                        fs.density[fluid::ix(i, j, size) as usize] // - 0.02,
                    } else {
                        0.5 * (fs.v_x[fluid::ix(i, j, size) as usize]
                            + fs.v_y[fluid::ix(i, j, size) as usize])
                            .abs()
                    }));

                    // let square = rectangle::square(
                    //                         x as f64,
                    //                         y as f64,
                    //                         scale as f64,
                    //                         )
                    let square =
                        rectangle::centered([x as f64, y as f64, scale as f64, scale as f64]);
                    let transform = _c.transform.trans(0.0, 0.0);

                    rectangle.draw(square, &_c.draw_state, transform, g);
                }
            }
        });
        if let Some(p) = e.press_args() {
            if p == Button::Mouse(MouseButton::Left) {
                dye = true;
                vel = true;
            }
            if p == Button::Mouse(MouseButton::Right) {
                vel = true;
            }
            if p == Button::Keyboard(Key::V) {
                vis = !vis;
            }
        }
        if let Some(p) = e.release_args() {
            if p == Button::Mouse(MouseButton::Left) {
                dye = false;
                vel = false;
            }
            if p == Button::Mouse(MouseButton::Right) {
                vel = false;
            }
        }

        if let Some(m) = e.mouse_cursor_args() {
            if dye {
                fs.add_dye(
                    //                  (e[0] as i32 / (scale * scale)) as i32,
                    //                  (e[1] as i32 / (scale * scale)) as i32,
                    (m[0] as i32 / scale) as i32,
                    (m[1] as i32 / scale) as i32,
                    dye_amount,
                );
            }
            if vel {
                let mut vel_add = 50.0;
                if dye {
                    vel_add = 20.0;
                }
                fs.add_velocity(
                    (m[0] as i32 / scale) as i32,
                    (m[1] as i32 / scale) as i32,
                    ((m[0] - prev_x) * vel_add) as f32,
                    ((m[1] - prev_y) * vel_add) as f32,
                );
                prev_x = m[0];
                prev_y = m[1];
            }
        }
    }
}
